#-------------------------------------------------------------------------------
# Name:        Command-line CRX Packer
# Purpose:     Pack Google Chrome extensions on the command line.
#
# Author:      jthuraisamy
# Created:     2013-11-01
#-------------------------------------------------------------------------------
#!/usr/bin/env python

# Import internal modules.
import os
import struct
import zipfile
import subprocess
from argparse import ArgumentParser

class CRX(object):
    def __init__(self, input_dir, key_file, output_file):
        self.input_dir = os.path.normpath(input_dir)
        self.key_file = os.path.normpath(key_file)
        self.output_file = os.path.normpath(output_file)
        self.zip_file = '%s.zip' % self.output_file

    def _run_process(self, cmd_line):
        return subprocess.Popen(cmd_line, stdout=subprocess.PIPE).stdout.read()

    def pack(self):
        # Zip input directory.
        with zipfile.ZipFile(self.zip_file, 'w') as zip_hnd:
            for directory in os.walk(self.input_dir):
                current_dir = directory[0]
                for filename in directory[2]:
                    file_path = os.path.join(current_dir, filename)
                    zip_path = os.path.join(os.path.relpath(current_dir,
                        self.input_dir), filename)
                    zip_hnd.write(file_path, zip_path, zipfile.ZIP_DEFLATED)
        # Generate public key from private key in key_file.
        pub_key = self._run_process(["openssl", "rsa", "-pubout", "-inform",
            "PEM", "-outform", "DER", "-in", self.key_file.replace('\\', '/')])
        # Sign zip_file with the key_file in PEM format.
        signature = self._run_process(["openssl", "sha1", "-sign",
            self.key_file.replace('\\', '/'),
            self.zip_file.replace('\\', '/')])
        # Generate header.
        magic_number = 'Cr24'
        version = struct.pack("<I", 2)
        pub_key_len = struct.pack("<I", len(pub_key))
        signature_len = struct.pack("<I", len(signature))
        # Write data to CRX file.
        with open(self.output_file, 'wb') as crx:
            crx.write(magic_number)
            crx.write(version)
            crx.write(pub_key_len)
            crx.write(signature_len)
            crx.write(pub_key)
            crx.write(signature)
            with open(self.zip_file, 'rb') as zip_file:
                crx.write(zip_file.read())
        # Delete intermediary zip.
        os.remove(self.zip_file)

def main():
    # Parse arguments.
    parser = ArgumentParser(description='CRX Packer by jthuraisamy (2013)')
    parser.add_argument('--_dir', required=True, help='Extension Directory')
    parser.add_argument('--key', required=True, help='Private Key (PEM file)')
    parser.add_argument('--crx', required=True, help='Output CRX')
    args = parser.parse_args()
    # Initialise CRX.
    crx = CRX(args.dir, args.key, args.crx)
    # Pack CRX.
    crx.pack()

if __name__ == '__main__':
    main()



import os
import zipfile
from crx3_pb2 import SignedData, CrxFileHeader
import struct
import io
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization, hashes
from cryptography.hazmat.primitives.asymmetric import rsa, utils, padding

fileBufferLength = 4096


def create_publickey(private_key):
    public_key = private_key.public_key()
    data = public_key.public_bytes(
        encoding=serialization.Encoding.DER,
        format=serialization.PublicFormat.SubjectPublicKeyInfo,
    )
    return data


def get_zipped_data_and_basename_from_dir(_dir):
    if os.path.isdir(_dir):
        return zipdir(_dir), _dir
    else:
        raise IOError('Source is not a directory or zip file <%s>' % _dir)


def package(_dir, output):
    zipdata, basename = get_zipped_data_and_basename_from_dir(_dir)

    private_key = rsa.generate_private_key(public_exponent=65537, key_size=2048, backend=default_backend())
    public_key = create_publickey(private_key)

    signed_header_data_str = create_signed_header_data_str(public_key)

    signed = sign(signed_header_data_str, zipdata, private_key)
    header_str = create_header_str(public_key, signed, signed_header_data_str)

    save_crx_file(header_str, zipdata, output, basename)


def sign(signed_header_data_str, zipped, private_key):
    signed_header_size_octets = struct.pack('<I', len(signed_header_data_str))

    chosen_hash = hashes.SHA256()
    hasher = hashes.Hash(chosen_hash, default_backend())
    hasher.update(b'CRX3 SignedData\00')
    hasher.update(signed_header_size_octets)
    hasher.update(signed_header_data_str)

    for i in range(0, len(zipped), fileBufferLength):
        if i + fileBufferLength <= len(zipped):
            hasher.update(zipped[i: i + fileBufferLength])
        else:
            hasher.update(zipped[i: len(zipped)])

    digest = hasher.finalize()

    return private_key.sign(digest, padding.PKCS1v15(), utils.Prehashed(chosen_hash))


def zipdir(directory, inject=None):
    zip_memory = io.BytesIO()
    with zipfile.ZipFile(zip_memory, 'w', zipfile.ZIP_DEFLATED) as zf:

        def _rec_zip(path, parent='', _inject=None):
            if _inject:
                for fname, fdata in _inject.items():
                    zf.writestr(fname, fdata)

            for d in os.listdir(path):
                child = os.path.join(path, d)
                name = '%s/%s' % (parent, d)
                if os.path.isfile(child):
                    zf.write(child, name)
                if os.path.isdir(child):
                    _rec_zip(child, name)

        _rec_zip(directory, _inject=inject)
        zf.close()
        zipdata = zip_memory.getvalue()
        return zipdata


def get_crx_id(public_key):
    digest = hashes.Hash(hashes.SHA256(), backend=default_backend())
    digest.update(public_key)
    hased = digest.finalize()
    return hased[0:16]


def create_signed_header_data_str(public_key):
    signed_header_data = SignedData()
    signed_header_data.crx_id = get_crx_id(public_key)
    return signed_header_data.SerializeToString()


def create_header_str(public_key, signed, signed_header_data_str):
    header = CrxFileHeader()
    proof = header.sha256_with_rsa.add()
    proof.public_key = public_key
    proof.signature = signed
    header.signed_header_data = signed_header_data_str
    return header.SerializeToString()


def save_crx_file(header_str, zipped, path, crdx):
    header_size_octets = struct.pack('<I', len(header_str))

    fileLocation = path if path else '%s.crx' % crdx
    with open(fileLocation, 'wb') as crx:
        data = [b'Cr24', struct.pack('<I', 3), header_size_octets, header_str, zipped]
        for d in data:
            crx.write(d)


def create(path_extension: str, path_save: str = ''):
    return package(path_extension, path_save)


create()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: crx3.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='crx3.proto',
  package='crx_file',
  syntax='proto2',
  serialized_options=_b('H\003'),
  serialized_pb=_b('\n\ncrx3.proto\x12\x08\x63rx_file\"\x9c\x01\n\rCrxFileHeader\x12\x35\n\x0fsha256_with_rsa\x18\x02 \x03(\x0b\x32\x1c.crx_file.AsymmetricKeyProof\x12\x37\n\x11sha256_with_ecdsa\x18\x03 \x03(\x0b\x32\x1c.crx_file.AsymmetricKeyProof\x12\x1b\n\x12signed_header_data\x18\x90N \x01(\x0c\";\n\x12\x41symmetricKeyProof\x12\x12\n\npublic_key\x18\x01 \x01(\x0c\x12\x11\n\tsignature\x18\x02 \x01(\x0c\"\x1c\n\nSignedData\x12\x0e\n\x06\x63rx_id\x18\x01 \x01(\x0c\x42\x02H\x03')
)




_CRXFILEHEADER = _descriptor.Descriptor(
  name='CrxFileHeader',
  full_name='crx_file.CrxFileHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sha256_with_rsa', full_name='crx_file.CrxFileHeader.sha256_with_rsa', index=0,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sha256_with_ecdsa', full_name='crx_file.CrxFileHeader.sha256_with_ecdsa', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signed_header_data', full_name='crx_file.CrxFileHeader.signed_header_data', index=2,
      number=10000, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=25,
  serialized_end=181,
)


_ASYMMETRICKEYPROOF = _descriptor.Descriptor(
  name='AsymmetricKeyProof',
  full_name='crx_file.AsymmetricKeyProof',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='public_key', full_name='crx_file.AsymmetricKeyProof.public_key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signature', full_name='crx_file.AsymmetricKeyProof.signature', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=183,
  serialized_end=242,
)


_SIGNEDDATA = _descriptor.Descriptor(
  name='SignedData',
  full_name='crx_file.SignedData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='crx_id', full_name='crx_file.SignedData.crx_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=244,
  serialized_end=272,
)

_CRXFILEHEADER.fields_by_name['sha256_with_rsa'].message_type = _ASYMMETRICKEYPROOF
_CRXFILEHEADER.fields_by_name['sha256_with_ecdsa'].message_type = _ASYMMETRICKEYPROOF
DESCRIPTOR.message_types_by_name['CrxFileHeader'] = _CRXFILEHEADER
DESCRIPTOR.message_types_by_name['AsymmetricKeyProof'] = _ASYMMETRICKEYPROOF
DESCRIPTOR.message_types_by_name['SignedData'] = _SIGNEDDATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CrxFileHeader = _reflection.GeneratedProtocolMessageType('CrxFileHeader', (_message.Message,), dict(
  DESCRIPTOR = _CRXFILEHEADER,
  __module__ = 'crx3_pb2'
  # @@protoc_insertion_point(class_scope:crx_file.CrxFileHeader)
  ))
_sym_db.RegisterMessage(CrxFileHeader)

AsymmetricKeyProof = _reflection.GeneratedProtocolMessageType('AsymmetricKeyProof', (_message.Message,), dict(
  DESCRIPTOR = _ASYMMETRICKEYPROOF,
  __module__ = 'crx3_pb2'
  # @@protoc_insertion_point(class_scope:crx_file.AsymmetricKeyProof)
  ))
_sym_db.RegisterMessage(AsymmetricKeyProof)

SignedData = _reflection.GeneratedProtocolMessageType('SignedData', (_message.Message,), dict(
  DESCRIPTOR = _SIGNEDDATA,
  __module__ = 'crx3_pb2'
  # @@protoc_insertion_point(class_scope:crx_file.SignedData)
  ))
_sym_db.RegisterMessage(SignedData)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Импорт недавно установленного пакета setuptools.
import setuptools

# Открытие README.md и присвоение его long_description.
with open("README.md", "r") as fh:
    long_description = fh.read()

# Определение requests как requirements для того, чтобы этот пакет работал. Зависимости проекта.
# requirements = ["requests<=2.21.0"]

# Функция, которая принимает несколько аргументов. Она присваивает эти значения пакету.
setuptools.setup(
    # Имя дистрибутива пакета. Оно должно быть уникальным, поэтому добавление вашего имени пользователя в конце является обычным делом.
    name="dir_to_crx",
    # Номер версии вашего пакета. Обычно используется семантическое управление версиями.
    version="1.0.3",
    # Имя автора.
    author="Aleksey Chernyshev",
    # Его почта.
    author_email="chernyshev.prog@gmail.com",
    # Краткое описание, которое будет показано на странице PyPi.
    description="Convert directory to .crx for google extension",
    # Длинное описание, которое будет отображаться на странице PyPi. Использует README.md репозитория для заполнения.
    long_description=long_description,
    # Определяет тип контента, используемый в long_description.
    long_description_content_type="text/markdown",
    # URL-адрес, представляющий домашнюю страницу проекта. Большинство проектов ссылаются на репозиторий.
    url="https://github.com/chernyshev-aleksey/dir_fo_crx",
    # Находит все пакеты внутри проекта и объединяет их в дистрибутив.
    packages=setuptools.find_packages(),
    # requirements или dependencies, которые будут установлены вместе с пакетом, когда пользователь установит его через pip.
    install_requires=[
        'cryptography>=37.0.2'
    ],
    # Предоставляет pip некоторые метаданные о пакете. Также отображается на странице PyPi.
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    # Требуемая версия Python.
    python_requires='>=3.6',
)